<?php
/**
 * @category    Pinpoint
 * @package     Pinpoint_Faq
 * @copyright   Copyright (c) 2019 Pinpoint Designs. (http://www.Pinpoint.co.uk)
 * @author      Ahmed Goda <hello@Pinpoint.co.uk>
 */

namespace Pinpoint\Faq\Controller\Adminhtml\Faq;

use Magento\Backend\App\Action\Context;
use Magento\Backend\Model\View\Result\Redirect as RedirectResult;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\Request\DataPersistorInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Registry;
use Pinpoint\Faq\Api\FaqRepositoryInterface;
use Pinpoint\Faq\Api\Data\FaqInterface;
use Pinpoint\Faq\Model\Faq;
use Pinpoint\Faq\Model\FaqFactory;
use Pinpoint\Faq\Model\ResourceModel\Faq as FaqResource;
use Pinpoint\Faq\Model\ResourceModel\FaqFactory as FaqResourceFactory;

/**
 * Class Save
 */
class Save extends \Pinpoint\Faq\Controller\Adminhtml\Faq implements HttpPostActionInterface
{
    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var FaqFactory
     */
    protected $faqFactory;

    /**
     * @var FaqRepositoryInterface
     */
    protected $faqRepository;

    /**
     * @var FaqResourceFactory
     */
    protected $faqResourceFactory;

    /**
    * @var \Magento\Framework\Mail\Template\TransportBuilder
    */
    protected $_transportBuilder;

    /**
    * @var \Magento\Framework\Translate\Inline\StateInterface
    */
    protected $inlineTranslation;

    /**
    * @var \Magento\Framework\App\Config\ScopeConfigInterface
    */
    protected $scopeConfig;

    /**
    * @var \Magento\Store\Model\StoreManagerInterface
    */
    protected $storeManager;

    /**
     * @var \Magento\Framework\Escaper
     */
    protected $_escaper;

    /**
     * Catalog product model
     *
     * @var \Magento\Catalog\Api\ProductRepositoryInterface
     */
    protected $productRepository;

    /**
     * @var \Magento\Framework\Mail\Template\TransportBuilder
     */
    protected $transportBuilder;

    /**
     * @param Context $context
     * @param Registry $coreRegistry
     * @param DataPersistorInterface $dataPersistor
     * @param FaqResourceFactory $faqResourceFactory
     * @param FaqFactory $faqFactory
     * @param FaqRepositoryInterface|null $faqRepository
     * @param \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
     * @param \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Catalog\Api\ProductRepositoryInterface $productRepository
     * @param \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder
     */
    public function __construct(
        Context $context,
        Registry $coreRegistry,
        DataPersistorInterface $dataPersistor,
        FaqResourceFactory $faqResourceFactory = null,
        FaqFactory $faqFactory = null,
        FaqRepositoryInterface $faqRepository = null,
        \Magento\Framework\Mail\Template\TransportBuilder $transportBuilder,
        \Magento\Framework\Translate\Inline\StateInterface $inlineTranslation,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Framework\Escaper $escaper,
        \Magento\Catalog\Api\ProductRepositoryInterface $productRepository
    ) {
        parent::__construct($context, $coreRegistry);
        $this->dataPersistor      = $dataPersistor;
        $this->faqResourceFactory = $faqResourceFactory ?: ObjectManager::getInstance()->get(FaqResourceFactory::class);
        $this->faqFactory         = $faqFactory ?: ObjectManager::getInstance()->get(FaqFactory::class);
        $this->faqRepository      = $faqRepository ?: ObjectManager::getInstance()->get(FaqRepositoryInterface::class);
        $this->_transportBuilder  = $transportBuilder;
        $this->inlineTranslation  = $inlineTranslation;
        $this->scopeConfig        = $scopeConfig;
        $this->storeManager       = $storeManager;
        $this->_escaper           = $escaper;
        $this->productRepository  = $productRepository;
        $this->transportBuilder   = $transportBuilder;
    }

    /**
     * @return RedirectResult
     */
    public function execute()
    {
        /** @var RedirectResult $redirectResult */
        $redirectResult = $this->resultRedirectFactory->create();

        $data = $this->getRequest()->getPostValue();

        if (!$data) {
            return $redirectResult->setPath('*/*/');
        }

        if (empty($data[FaqInterface::FAQ_ID])) {
            $data[FaqInterface::FAQ_ID] = null;
        }

        $faq = $this->createModel();

        $faqId = $this->getRequest()->getParam(FaqInterface::FAQ_ID);

        if ($faqId) {
            try {
                $faq = $this->faqRepository->getById($faqId);
            } catch (LocalizedException $e) {
                $this->messageManager->addErrorMessage(
                    __('This question no longer exists.')
                );
                return $redirectResult->setPath('*/*/');
            }
        }
        // check if question answered to publish and send an email to customer
        // better to be replaced with helper to not violate single responsibility of controller
        if (!empty($data['answer_text'])) {
            $data['is_active'] = true;
            if (!empty($data['is_customer_notified'])) {
                $this->notifyCustomer($data);
            }
        }

        $faq->setData($data);

        try {
            $this->createResource()->save($faq);
            $this->messageManager->addSuccessMessage(__('You saved the FAQ.'));
            $this->dataPersistor->clear('pinpoint_faq');
            return $this->processReturnAction($faq, $data, $redirectResult);
        } catch (LocalizedException $e) {
            $this->messageManager->addErrorMessage($e->getMessage());
        } catch (\Exception $e) {
            $this->messageManager->addExceptionMessage($e, __('Something went wrong while saving the FAQ.'));
        }

        $this->dataPersistor->set('pinpoint_faq', $data);
        return $redirectResult->setPath('*/*/edit', [FaqInterface::FAQ_ID => $faqId]);
    }

    /**
     * @param  array $data
     * @return void
     * @throws Exception
     */
    protected function notifyCustomer(array $data = []): void
    {
        try {
            $transport = $this->transportBuilder->setTemplateIdentifier('emails_general_notification')
                ->setTemplateOptions(
                    [
                        'area' => \Magento\Framework\App\Area::AREA_FRONTEND,
                        'store' => 0,
                    ]
                )
                ->setTemplateVars(
                    [
                        'current_product' => $this->productRepository->getById($data['product_id'], false, $this->storeManager->getStore()->getId()),
                        'base_url'        => $this->storeManager->getStore()->getBaseUrl(),
                        'store_name'      => $this->storeManager->getStore()->getName(),
                        'website_name'    => $this->storeManager->getWebsite()->getName()
                    ]
                )
                ->setFrom('general')
                ->addTo(strip_tags($data['customer_email']))
                ->getTransport();

            $this->inlineTranslation->suspend();
            $transport->sendMessage();
        } catch (\Exception $exception) {
            \Magento\Framework\App\ObjectManager::getInstance()->get('Psr\Log\LoggerInterface')
                ->debug('Failed to send email due to: ' . $exception->getMessage());
        } finally {
            $this->inlineTranslation->resume();
        }
    }

    /**
     * @param Faq $faq
     * @param array $data
     * @param RedirectResult $resultRedirect
     * @return RedirectResult
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     */
    protected function processReturnAction(
        Faq $faq,
        array $data,
        RedirectResult $resultRedirect
    ): RedirectResult {
        $redirect = $data['back'] ?? 'close';

        if ($redirect === 'continue') {
            $resultRedirect->setPath(
                '*/*/edit',
                [
                FaqInterface::FAQ_ID => $faq->getId(),
                ]
            );
        } elseif ($redirect === 'close') {
            $resultRedirect->setPath('*/*/');
        } elseif ($redirect === 'duplicate') {
            $duplicateModel = $this->createModel(['data' => $data]);
            $duplicateModel->setId(null);
            $this->createResource()->save($duplicateModel);
            $id = $duplicateModel->getId();
            $this->messageManager->addSuccessMessage(
                __('You duplicated the FAQ.')
            );
            $this->dataPersistor->set('pinpoint_faq', $data);
            $resultRedirect->setPath('*/*/edit', [FaqInterface::FAQ_ID => $id]);
        }
        return $resultRedirect;
    }

    /**
     * @param array $data
     * @return Faq
     */
    protected function createModel(array $data = []): Faq
    {
        return  $this->faqFactory->create($data);
    }

    /**
     * @return FaqResource
     */
    protected function createResource(): FaqResource
    {
        return $this->faqResourceFactory->create();
    }
}