<?php
/**
 * @category    Pinpoint
 * @package     Pinpoint_Faq
 * @copyright   Copyright (c) 2019 Pinpoint Designs. (http://www.Pinpoint.co.uk)
 * @author      Ahmed Goda <hello@Pinpoint.co.uk>
 */

namespace Pinpoint\Faq\Ui\Component\Listing\Column;

use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\UiComponent\ContextInterface;
use Magento\Framework\View\Element\UiComponentFactory;
use Magento\Ui\Component\Listing\Columns\Column;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\Escaper;
use Pinpoint\Faq\Api\Data\FaqInterface;

/**
 * Class to build edit and delete link for each item.
 */
class Actions extends Column
{
    /**
     * Url path
     */
    const URL_PATH_EDIT = 'pinpoint_faqs/faq/edit';
    const URL_PATH_DELETE = 'pinpoint_faqs/faq/delete';

    /**
     * @var UrlInterface
     */
    protected $urlBuilder;

    /**
     * @var Escaper
     */
    private $escaper;

    /**
     * Actions constructor
     *
     * @param ContextInterface $context
     * @param UiComponentFactory $uiComponentFactory
     * @param UrlInterface $urlBuilder
     * @param array $components
     * @param array $data
     */
    public function __construct(
        ContextInterface   $context,
        UiComponentFactory $uiComponentFactory,
        UrlInterface       $urlBuilder,
        array              $components = [],
        array              $data = []
    ) {
        $this->urlBuilder = $urlBuilder;
        parent::__construct($context, $uiComponentFactory, $components, $data);
    }

    /**
     * @inheritDoc
     */
    public function prepareDataSource(array $dataSource)
    {
        if (!isset($dataSource['data']['items'])) {
            return $dataSource;
        }
        foreach ($dataSource['data']['items'] as & $item) {
            if (!isset($item[FaqInterface::FAQ_ID])) {
                return null;
            }
            $title = $this->getEscaper()->escapeHtmlAttr(
                $item[FaqInterface::QUESTION_TEXT]
            );
            $item[$this->getData('name')] = [
                'edit' => [
                    'href' => $this->prepareEditUrl($item),
                    'label' => __('Edit'),
                ],
                'delete' => [
                    'href' => $this->prepareDeleteUrl($item),
                    'label' => __('Delete'),
                    'confirm' => [
                        'title' => __('Delete %1', $title),
                        'message' => __('Are you sure you want to delete a %1 record?', $title),
                    ],
                    'post' => true,
                ],
            ];
        }
        return $dataSource;
    }

    /**
     * Get instance of escaper
     *
     * @return Escaper
     * @deprecated 101.0.7
     */
    private function getEscaper()
    {
        if (!$this->escaper) {
            $this->escaper = ObjectManager::getInstance()->get(Escaper::class);
        }
        return $this->escaper;
    }

    /**
     * @param $item
     * @return string
     */
    protected function prepareEditUrl($item): string
    {
        return $this->urlBuilder->getUrl(
            static::URL_PATH_EDIT,
            [
                FaqInterface::FAQ_ID => $item[FaqInterface::FAQ_ID],
            ]
        );
    }

    /**
     * @param $item
     * @return string
     */
    protected function prepareDeleteUrl($item): string
    {
        return $this->urlBuilder->getUrl(
            static::URL_PATH_DELETE,
            [
                FaqInterface::FAQ_ID => $item[FaqInterface::FAQ_ID],
            ]
        );
    }
}

